@@ -1,12 +1,12 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool-II/pool_type.h,v 1.3.2.1 2007/07/24 01:57:27 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_type.h,v 1.11 2009/12/06 12:55:08 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2007	PgPool Global Development Group
+ * Copyright (c) 2003-2009	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -30,6 +30,20 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 
+typedef signed char int8;		/* == 8 bits */
+typedef signed short int16;		/* == 16 bits */
+typedef signed int int32;		/* == 32 bits */
+
+/*
+ * uintN
+ *		Unsigned integer, EXACTLY N BITS IN SIZE,
+ *		used for numerical computations and the
+ *		frontend/backend protocol.
+ */
+typedef unsigned char uint8;	/* == 8 bits */
+typedef unsigned short uint16;	/* == 16 bits */
+typedef unsigned int uint32;	/* == 32 bits */
+
 /* Define common boolean type. C++ and BEOS already has it so exclude them. */
 #ifdef c_plusplus
 #ifndef __cplusplus
@@ -69,6 +83,7 @@ typedef char bool;
 #define MAX_NUM_BACKENDS 128
 #define MAX_CONNECTION_SLOTS 128
 #define MAX_DB_HOST_NAMELEN	 128
+#define MAX_PATH_LENGTH 256
 
 typedef enum {
 	CON_UNUSED,		/* unused slot */
@@ -90,6 +105,8 @@ typedef struct {
 	int backend_port;	/* backend port numbers */
 	BACKEND_STATUS backend_status;	/* backend status */
 	double backend_weight;	/* normalized backend load balance ratio */
+	double unnormalized_weight; /* descripted parameter */
+	char backend_data_directory[MAX_PATH_LENGTH];
 } BackendInfo;
 
 typedef struct {
@@ -105,9 +122,11 @@ typedef struct {
 	char		user[SM_USER];	/* User name */
 	int			major;	/* protocol major version */
 	int			minor;	/* protocol minor version */
+	int			pid;	/* backend process id */
+	int			key;	/* cancel key */
 	int			counter; /* used counter */
 	time_t 		create_time; /* connection creation time */
-	int			load_balancing_node; /*load balancing node */
+	int load_balancing_node; /* load balancing node */
 } ConnectionInfo;
 
 
@@ -118,7 +137,7 @@ typedef struct {
 typedef struct {
 	pid_t pid; /* OS's process id */
 	time_t start_time; /* fork() time */
-	ConnectionInfo *connection_info; /* connection information */
+	ConnectionInfo *connection_info; /* head of the connection info for this process */
 } ProcessInfo;
 
 /*
@@ -139,24 +158,44 @@ typedef struct {
 	int is_created_prepare;	/* is prepare statement created? */
 } DistDefInfo;
 
+typedef struct {
+	char *dbname;     /* database name */
+	char *schema_name;    /* schema name */
+	char *table_name;   /* table name */
+	int  col_num;     /* number of clumn*/
+	char **col_list;    /* column list */
+	char **type_list;   /* type list */
+	char *prepare_name;   /* prepared statement name */
+	int is_created_prepare; /* is prepare statement created? */
+} RepliDefInfo;
+
 typedef struct {
 	int has_prepared_statement;	/* true if the current session has prepared statement created */
 	char *register_prepared_statement; /* prepared statement name for cache register */
 } QueryCacheTableInfo;
 
 typedef struct {
-	char *hostname;			/* host name */
-	int port;				/* port number */
-	char *user;				/* login user name */
-	char *password;			/* login password */
-	char *schema_name;		/* schema name */
-	char *database_name;	/* database name */
-	int dist_def_num;		/* number of distribution table */
+	char *hostname;     /* host name */
+	int port;       /* port number */
+	char *user;       /* login user name */
+	char *password;     /* login password */
+	char *schema_name;    /* schema name */
+	char *database_name;  /* database name */
+	int repli_def_num;    /* number of replication table */
+	int dist_def_num;   /* number of distribution table */
+	RepliDefInfo *repli_def_slot; /* replication rule list */
 	DistDefInfo *dist_def_slot; /* distribution rule list */
 	QueryCacheTableInfo query_cache_table_info; /* query cache db session info */
 	BACKEND_STATUS system_db_status;
 } SystemDBInfo;
 
+/*
+ * Backend status record file
+ */
+typedef struct {
+	BACKEND_STATUS status[MAX_NUM_BACKENDS];
+} BackendStatusRecord;
+
 /*
  *  It seems that sockaddr_storage is now commonly used in place of sockaddr.
  *  So, define it if it is not define yet, and create new SockAddr structure
